import { fileProxy, httpProxy, createApiProxy, wsProxy } from '@/module/assistant/index.ts';
import http from 'http';
import { LocalProxy } from './local-proxy.ts';
import { assistantConfig, app } from '@/app.ts';
import { log } from '@/module/logger.ts';
const localProxy = new LocalProxy({
  assistantConfig,
});
export const proxyRoute = async (req: http.IncomingMessage, res: http.ServerResponse) => {
  const _assistantConfig = assistantConfig.getCacheAssistantConfig();
  const appDir = assistantConfig.configPath?.pagesDir;
  const url = new URL(req.url, 'http://localhost');
  const pathname = url.pathname;
  if (pathname === '/' && _assistantConfig?.home) {
    res.writeHead(302, { Location: `${_assistantConfig?.home}/` });
    return res.end();
  }
  if (pathname.startsWith('/favicon.ico')) {
    res.statusCode = 404;
    res.end('Not Found Favicon');
    return;
  }
  if (pathname.startsWith('/client')) {
    console.debug('handle by router');
    return;
  }
  // client, api, v1, serve 开头的拦截
  const apiProxyList = _assistantConfig?.apiProxyList || [];
  const defaultApiProxy = createApiProxy(_assistantConfig?.pageApi || 'https://kevisual.cn');
  const apiBackendProxy = [...apiProxyList, ...defaultApiProxy].find((item) => pathname.startsWith(item.path));
  if (apiBackendProxy) {
    log.debug('apiBackendProxy', { apiBackendProxy, url: req.url });
    return httpProxy(req, res, {
      path: apiBackendProxy.path,
      target: apiBackendProxy.target,
    });
  }
  const urls = pathname.split('/');
  const [_, _user, _app] = urls;
  if (!_app) {
    res.statusCode = 404;
    res.end('Not Found Proxy');
    return;
  }
  if (_app && urls.length === 3) {
    // 重定向到
    res.writeHead(302, { Location: `${req.url}/` });
    return res.end();
  }
  const proxyApiList = _assistantConfig?.proxy || [];
  const proxyApi = proxyApiList.find((item) => pathname.startsWith(item.path));
  if (proxyApi && proxyApi.type === 'file') {
    log.debug('proxyApi', { proxyApi, pathname });
    const _indexPath = proxyApi.indexPath || `${_user}/${_app}/index.html`;
    const _rootPath = proxyApi.rootPath;
    if (!_rootPath) {
      log.error('Not Found rootPath', { proxyApi, pathname });
      return res.end(`Not Found [${proxyApi.path}] rootPath`);
    }
    return fileProxy(req, res, {
      path: proxyApi.path, // 代理路径, 比如/root/center
      rootPath: proxyApi.rootPath,
      ...proxyApi,
      indexPath: _indexPath, // 首页路径
    });
  }
  const localProxyProxyList = localProxy.getLocalProxyList();
  const localProxyProxy = localProxyProxyList.find((item) => pathname.startsWith(item.path));
  if (localProxyProxy) {
    log.log('localProxyProxy', { localProxyProxy, url: req.url });
    return fileProxy(req, res, {
      path: localProxyProxy.path,
      rootPath: appDir,
      indexPath: localProxyProxy.indexPath,
    });
  }
  const creatCenterProxy = createApiProxy(_assistantConfig?.pageApi || 'https://kevisual.cn', ['/root']);
  const centerProxy = creatCenterProxy.find((item) => pathname.startsWith(item.path));
  if (centerProxy) {
    return httpProxy(req, res, {
      path: centerProxy.path,
      target: centerProxy.target,
      type: 'http',
    });
  }
  log.debug('handle by router 404', req.url);

  res.statusCode = 404;
  res.end('Not Found Proxy');
};

export const proxyWs = () => {
  const apiProxyList = assistantConfig.getCacheAssistantConfig()?.apiProxyList || [];
  const proxy = assistantConfig.getCacheAssistantConfig()?.proxy || [];
  const proxyApi = [...apiProxyList, ...proxy].filter((item) => item.ws);
  log.debug('proxyApi ', proxyApi);
  wsProxy(app.server.server, {
    apiList: proxyApi,
  });
};
